package com.jiuqi.util;

import java.text.DecimalFormat;

public class DataUtil
{
    public static final String DATA_NULL_VALUE = "--";
    public static final int PERCENT_DECIMAL = 1;
    
    public static int compare(final Object o1, final Object o2) {
        if (o1 == null && o2 == null) {
            return 0;
        }
        if (o1 == null) {
            return -1;
        }
        if (o2 == null) {
            return 1;
        }
        final double d1 = (double)o1;
        final double d2 = (double)o2;
        if (d1 < d2) {
            return -1;
        }
        if (d1 > d2) {
            return 1;
        }
        return 0;
    }
    
    public static String getDigitValue(final int demical, final double src) {
        final DecimalFormat df = new DecimalFormat();
        final StringBuffer buff = new StringBuffer();
        buff.append("#");
        if (demical > -1) {
            if (demical > 0) {
                buff.append("0.");
            }
            else {
                buff.append("#");
            }
            for (int i = 0; i < demical; ++i) {
                buff.append("0");
            }
        }
        else {
            buff.append("#.#####");
        }
        df.applyPattern(buff.toString());
        return df.format(src);
    }
    
    public static boolean isDataValue(final Object value) {
        return value != null && (value instanceof Double || value instanceof Integer || value instanceof Float || value instanceof Long || value instanceof Short);
    }
    
    public static Double divide(final Object o1, final Object o2) {
        if (o1 == null) {
            return new Double(0.0);
        }
        if (o2 == null) {
            return null;
        }
        if (!isDataValue(o1)) {
            return null;
        }
        final double v1 = (double)o1;
        final double v2 = (double)o2;
        if (v2 == 0.0) {
            return null;
        }
        return new Double(v1 / v2);
    }
    
    public static Object calcSum(final Object[] values) {
        if (values == null || values.length == 0) {
            return null;
        }
        Object validObj = null;
        for (int i = 0; i < values.length; ++i) {
            if (values[i] != null) {
                validObj = values[i];
                break;
            }
        }
        if (validObj == null) {
            return null;
        }
        if (validObj instanceof Double) {
            double returnValue = 0.0;
            for (int j = 0; j < values.length; ++j) {
                if (values[j] != null) {
                    returnValue += (double)values[j];
                }
            }
            return new Double(returnValue);
        }
        if (validObj instanceof Integer || validObj instanceof Long || validObj instanceof Short) {
            long returnValue2 = 0L;
            for (int j = 0; j < values.length; ++j) {
                if (values[j] != null) {
                    returnValue2 += (int)values[j];
                }
            }
            return new Long(returnValue2);
        }
        if (validObj instanceof Float) {
            float returnValue3 = 0.0f;
            for (int k = 0; k < values.length; ++k) {
                if (values[k] != null) {
                    returnValue3 += (float)values[k];
                }
            }
            return new Float(returnValue3);
        }
        return "";
    }
    
    public static int[] calcPlace(final Object[] values) {
        if (values == null) {
            return null;
        }
        if (values.length == 0) {
            return new int[0];
        }
        final int[] returnPlace = new int[values.length];
        for (int i = 0; i < values.length; ++i) {
            int p = 1;
            for (int j = 0; j < values.length; ++j) {
                if (j != i) {
                    if (compare(values[j], values[i]) > 0) {
                        ++p;
                    }
                }
            }
            returnPlace[i] = p;
        }
        return returnPlace;
    }
    
    public static boolean isLessthanOrEqualZeroData(final Object value) {
        if (isNegativeData(value)) {
            return true;
        }
        if (!isDataValue(value)) {
            return true;
        }
        final double d = (double)value;
        return d == 0.0;
    }
    
    public static boolean isNegativeData(final Object value) {
        if (!isDataValue(value)) {
            return true;
        }
        final double d = (double)value;
        return d < 0.0;
    }
    
    public static boolean isZeroData(final Object value) {
        if (!isDataValue(value)) {
            return false;
        }
        final double d = (double)value;
        return d == 0.0;
    }
    
    public static String[] calcPercent(final Object[] values) {
        if (values == null) {
            return null;
        }
        if (values.length == 0) {
            return new String[0];
        }
        final String[] returnPercent = new String[values.length];
        for (int i = 0; i < values.length; ++i) {
            if (isNegativeData(values[i])) {
                for (int j = 0; j < values.length; ++j) {
                    returnPercent[j] = "--";
                }
                return returnPercent;
            }
        }
        final Object sumValue = calcSum(values);
        if (sumValue != null) {
            if (isLessthanOrEqualZeroData(sumValue)) {
                for (int k = 0; k < values.length; ++k) {
                    returnPercent[k] = "--";
                }
                return returnPercent;
            }
            int specialPercentIndex = -1;
            double otherPercentValue = 0.0;
            boolean notSpecialDealed = true;
            Object value = null;
            for (int l = 0; l < values.length; ++l) {
                value = values[l];
                if (value == null) {
                    returnPercent[l] = "--";
                }
                else if (!isZeroData(value) && notSpecialDealed) {
                    specialPercentIndex = l;
                    notSpecialDealed = false;
                }
                else {
                    value = divide(value, sumValue);
                    try {
                        final double percent = 100.0 * (double)value;
                        final String percentStr = getDigitValue(1, percent);
                        returnPercent[l] = String.valueOf(percentStr) + "%";
                        try {
                            otherPercentValue += Double.parseDouble(percentStr);
                        }
                        catch (final NumberFormatException ex) {}
                    }
                    catch (final Exception ex2) {}
                }
            }
            if (specialPercentIndex >= 0) {
                returnPercent[specialPercentIndex] = String.valueOf(getDigitValue(1, 100.0 - otherPercentValue)) + "%";
            }
        }
        return returnPercent;
    }
}
